@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -109,12 +109,27 @@ def quiet_raise(exc):
     def zero_byte_sequence(n):
         return bytes(n)
 
+    import struct
+
+    struct_pack = struct.pack
+    struct_unpack = struct.unpack
+
     from time import monotonic as benchmark_time
 else:
     import struct
 
+    def struct_pack(*args):
+        return hbytes(struct.pack(*args))
+
+    if CAN_UNPACK_BYTE_ARRAY:
+        def struct_unpack(fmt, string):
+            return struct.unpack(fmt, string)
+    else:
+        def struct_unpack(fmt, string):
+            return struct.unpack(fmt, str(string))
+
     def zero_byte_sequence(n):
-        return b'\0' * n
+        return hbytes(b'\0' * n)
 
     def int_from_bytes(data):
         assert isinstance(data, bytearray)
@@ -147,7 +162,7 @@ else:
         return hbytes(result)
 
     def bytes_from_list(ls):
-        return bytes(bytearray(ls))
+        return hbytes(bytearray(ls))
 
     def to_bytes_sequence(ls):
         return bytearray(ls)
@@ -247,6 +262,7 @@ def qualname(f):
     except AttributeError:
         return f.__name__
 
+
 FakeArgSpec = namedtuple(
     'ArgSpec', ('args', 'varargs', 'keywords', 'defaults'))
 
@@ -389,9 +405,11 @@ class compatbytes(bytearray):
         raise ValueError('Value %r not in sequence %r' % (value, self))
 
     def __add__(self, value):
+        assert isinstance(value, compatbytes)
         return compatbytes(bytearray.__add__(self, value))
 
     def __radd__(self, value):
+        assert isinstance(value, compatbytes)
         return compatbytes(bytearray.__add__(value, self))
 
     def __mul__(self, value):
@@ -422,6 +440,7 @@ class compatbytes(bytearray):
     def __contains__(self, value):
         return any(v == value for v in self)
 
+
 if PY2:
     hbytes = compatbytes
     reasonable_byte_type = bytearray
@@ -432,6 +451,8 @@ else:
     string_types = (str,)
 
 
+EMPTY_BYTES = hbytes(b'')
+
 if PY2:
     def to_str(s):
         if isinstance(s, unicode):
@@ -452,6 +473,7 @@ def cast_unicode(s, encoding=None):
 def get_stream_enc(stream, default=None):
     return getattr(stream, 'encoding', None) or default
 
+
 if PY3:
     FileNotFoundError = FileNotFoundError
 else:
