@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2016 David R. MacIver
+# Most of this work is copyright (C) 2013-2017 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -31,9 +31,9 @@ from functools import wraps
 
 from hypothesis.configuration import storage_directory
 from hypothesis.vendor.pretty import pretty
-from hypothesis.internal.compat import hrange, to_str, qualname, \
-    getargspec, to_unicode, isidentifier, str_to_bytes, \
-    ARG_NAME_ATTRIBUTE, update_code_location
+from hypothesis.internal.compat import ARG_NAME_ATTRIBUTE, hrange, \
+    to_str, qualname, getargspec, to_unicode, isidentifier, str_to_bytes, \
+    update_code_location
 
 
 def fully_qualified_name(f):
@@ -131,9 +131,9 @@ def convert_positional_arguments(function, args, kwargs):
 
     """
     argspec = getargspec(function)
-    kwargs = dict(kwargs)
+    new_kwargs = dict(kwargs)
     if not argspec.keywords:
-        for k in kwargs.keys():
+        for k in new_kwargs.keys():
             if k not in argspec.args:
                 raise TypeError(
                     '%s() got an unexpected keyword argument %r' % (
@@ -156,16 +156,16 @@ def convert_positional_arguments(function, args, kwargs):
         )
 
     for arg, name in zip(args, argspec.args):
-        if name in kwargs:
+        if name in new_kwargs:
             raise TypeError(
                 '%s() got multiple values for keyword argument %r' % (
                     function.__name__, name
                 ))
         else:
-            kwargs[name] = arg
+            new_kwargs[name] = arg
     return (
         tuple(args[len(argspec.args):]),
-        kwargs,
+        new_kwargs,
     )
 
 
@@ -190,7 +190,7 @@ LINE_CONTINUATION = re.compile(r"\\\n")
 WHITESPACE = re.compile(r"\s+")
 PROBABLY_A_COMMENT = re.compile("""#[^'"]*$""")
 SPACE_FOLLOWS_OPEN_BRACKET = re.compile(r"\( ")
-SPACE_PRECEDES_CLOSE_BRACKET = re.compile(r"\( ")
+SPACE_PRECEDES_CLOSE_BRACKET = re.compile(r" \)")
 
 
 def extract_lambda_source(f):
@@ -365,9 +365,9 @@ def accept(%(funcname)s):
 """.strip() + '\n'
 
 
-def copy_argspec(name, argspec):
-    """A decorator which sets the name and argspec of the function passed into
-    it."""
+def define_function_signature(name, docstring, argspec):
+    """A decorator which sets the name, argspec and docstring of the function
+    passed into it."""
     check_valid_identifier(name)
     for a in argspec.args:
         check_valid_identifier(a)
@@ -425,6 +425,7 @@ def copy_argspec(name, argspec):
             }).accept
 
         result = base_accept(f)
+        result.__doc__ = docstring
         result.__defaults__ = argspec.defaults
         return result
     return accept
@@ -453,5 +454,6 @@ def impersonate(target):
 def proxies(target):
     def accept(proxy):
         return impersonate(target)(wraps(target)(
-            copy_argspec(target.__name__, getargspec(target))(proxy)))
+            define_function_signature(
+                target.__name__, target.__doc__, getargspec(target))(proxy)))
     return accept
